// VDP Status register bit test
// Tests the sprite coincidence and vertical blank bits against polling
// Belief is that the VBLANK bit is racey against external polls

// uses libti99
#include "vdp.h"
int miss_blank=0, miss_sprite=0;

void fast_hex_out(unsigned int x) {
	unsigned int z;		// compiler bug: if this is unsigned char, the second byte is not masked correctly
	z=(x>>8);
	unsigned int dat = byte2hex[z];
	VDPWD = dat>>8;
	VDPWD = dat&0xff;

	z=(x&0xff);
	dat = byte2hex[z];
	VDPWD = dat>>8;
	VDPWD = dat&0xff;
}

void writeblank() {
	VDP_SET_ADDRESS_WRITE(VDP_SCREEN_POS(0,13));
	fast_hex_out(miss_blank);
}

void writesprite() {
	VDP_SET_ADDRESS_WRITE(VDP_SCREEN_POS(1,13));
	fast_hex_out(miss_sprite);
}

int main() {
	// init the screen
	{
		int x = set_graphics(VDP_SPR_8x8);		// set graphics mode 
		charsetlc();							// get character set including lowercase
		vdpmemset(gColor, 0x10, 32);			// all colors to black on transparent
		hchar(0,0,32,768);						// clear screen
		sprite(0,33,3,100,128);
		sprite(1,65,4,106,129);					// sprites touching near center of screen - only 1 scanline may collide though!
		VDP_SET_REGISTER(VDP_REG_COL, 0x07);	// background color
		VDP_SET_REGISTER(VDP_REG_MODE1, x);		// enable the display
		VDP_REG1_KSCAN_MIRROR = x;				// must have a copy of VDP Reg 1 if you ever use KSCAN
	}

	// text at the top for counting
	writestring(0,0,"Miss blank :");
	writestring(1,0,"Miss sprite:");
	writeblank();
	writesprite();

	// wait for a vblank before we start
	VDP_WAIT_VBLANK;

	for (;;) {
		// next flag set should be sprites
		unsigned char x;
		do {
			x=VDPST;
			if (x&VDP_ST_INT) {
				++miss_sprite;
				writesprite();
			}
		} while (0 == (x&VDP_ST_COINC));

		// next flag is vblank
		do {
			x=VDPST;
			if (x&VDP_ST_COINC) {
				++miss_blank;
				writeblank();
			}
		} while (0 == (x&VDP_ST_INT));
	}

	return 0;
}
